package com.windofkeltia.processor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import static java.util.Objects.nonNull;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestName;

import org.apache.nifi.util.MockFlowFile;
import org.apache.nifi.util.TestRunner;
import org.apache.nifi.util.TestRunners;

import com.windofkeltia.utilities.StringUtilities;
import com.windofkeltia.utilities.TestUtilities;

public class FilterTest
{
  @Rule   public TestName name = new TestName();
  @Before public void setUp() { TestUtilities.setUp( name ); runner = TestRunners.newTestRunner( new Filter() ); }
  @After  public void tearDown() { }

  private static final boolean VERBOSE  = true;//TestUtilities.VERBOSE;
  private static final int     ONE      = 1;

  private TestRunner runner;

  @Test
  public void testWithProperty() throws IOException
  {
    final String CONTENT = TestUtilities.getLinesFromFile( "exercise.txt" );

    displayOriginal( CONTENT );

    // how you create a dynamic property running under the NiFi JUnit test framework...
    runner.setValidateExpressionUsage( false );
    runner.setProperty( "quick brown", "wily red" );
    runner.setProperty( "ant",         "flea" );

    MockFlowFile flowfile = runTest( CONTENT );

    displayAttributes( flowfile );
    displayOutput( flowfile );
  }

  @Test
  public void testSansProperty()
  {
    final String CONTENT  = "This is a test of the Emergency Broadcast System. It is only a test.";

    displayOriginal( CONTENT );

    MockFlowFile flowfile = runTest( CONTENT );

    if( nonNull( flowfile ) )
    {
      displayAttributes( flowfile );
      displayOutput( flowfile );
    }
    else
    {
      System.out.println( "Content:\n  (There were no replacement strings, so the flowfile remains the same.)" );
    }
  }

  private void displayAttributes( MockFlowFile flowfile )
  {
    if( VERBOSE )
    {
      Map< String, String > attributes = flowfile.getAttributes();
      System.out.println( "Attributes:" );
      int keyWidth = 0;
      for( Map.Entry< String, String > attribute : attributes.entrySet() )
        keyWidth = Math.max( keyWidth, attribute.getKey().length() );
      for( Map.Entry< String, String > attribute : attributes.entrySet() )
        System.out.println( "  " + StringUtilities.padStringLeft( attribute.getKey(), keyWidth )
                          + " = " + attribute.getValue() );
    }
  }

  private void displayOriginal( final String content )
  {
    if( VERBOSE )
    {
      System.out.print  ( "\nOriginal content:\n  " );
      System.out.println( content.replace( "\n", "\n  " ) );
    }
  }

  private void displayOutput( MockFlowFile flowfile )
  {
    if( VERBOSE )
    {
      String content = new String( runner.getContentAsByteArray( flowfile ) );
      System.out.print  ( "\nContent:\n  " );
      System.out.println( content.replace( "\n", "\n  " ) );
    }
  }

  private MockFlowFile runTest( final String content )
  {
    runner.setValidateExpressionUsage( false );
    runner.enqueue( new ByteArrayInputStream( content.getBytes() ) );
    runner.run( ONE );
    runner.assertQueueEmpty();

    List< MockFlowFile > flowfiles = runner.getFlowFilesForRelationship( Filter.SUCCESS );

    if( flowfiles.size() < 1 )
      return null;

    return flowfiles.get( 0 );
  }
}